#ifndef bqtPicTypesHH
#define bqtPicTypesHH

#include <utility>

// Array utilities
template<typename T, std::size_t Size>
constexpr inline std::size_t count(T (&)[Size]) { return Size; }

// Bitfield utilities
template<unsigned bitno, unsigned bitcount, typename T>
struct RegBitSet
{
    T data;

    /* Bitmask is pow(2, bitcount) - 1. Avoid overflow if bitcount=64. */
    enum { mask = ((1ull << (bitcount/2)) << (bitcount-(bitcount/2))) - 1ull };

    operator T() const { return (data >> bitno) & mask; }

    template<typename T2>
    RegBitSet& operator=(T2 val)
    {
        /* Poke the given value into the data.
         * If bitcount=1, cast val as bool before doing the work.
         */
        data = T(data & T(~(mask << bitno)))
             | T((bitcount > 1 ? T(val & mask) : T(bool(val))) << bitno);
        return *this;
    }
};

#endif



